% Posterior kernel plot for sigma with normal and inverse gamma prior


clear all
clc
close all
mkdir('./results')
addpath('./tools')


randn('state',999);
T     = 1500;
k     = zeros(T+1,4);
z     = zeros(T+1,4);
rho   = 0.95;
alpha = 0.33;
beta  = 0.99;
sigma(1) = 0.00;
sigma(2) = 0.05;
sigma(3) = 0.10;
sigma(4) = 0.30;
error    = randn(T+1,4); 
sigme    = mean(sigma);
randn('state',999);
em    = sigme*randn(T+1,4); 



for e =1:4
    for t = 2 : T+1,
        z(t,e) = rho   * z(t-1,e) + sigma(e) * error(t,e);
        k(t,e) = alpha * k(t-1,e) + z(t,e);
        y(t,e) = z(t,e) + alpha * k(t-1,e) + em(t,e);
    end
%     subplot(4,1,e)
%     plot(y(:,e),'b'); hold on;
%     plot(k(:,e),'r');hold on;
%     plot(z(:,e),'k'); axis tight;
end

k(1:1400,:)= []; 
y(1:1400,:)= []; 
z(1:1400,:)= []; 

save data 

% plot the posterior kernel 
FF         = 50;
sig_igamma = linspace(0,0.5,FF);
sig_norm   = linspace(-0.5,0.5,FF);
sig_expo   = linspace(0,0.5,FF);

colors(2,:) =[0.00,0.00,0.90];
colors(1,:) =[0.45,0.85,0.15];
colors(3,:) =[0.90,0.00,0.00];


for e = 1 : 4 
    si     = 5; % inverse gamma
    sn     = 5; % normal 
    mi     = 0.2;% + sigma(e);
    mn     = 0.2;
    lambda = 5; %exponential


    for f= 1 :FF
        %%=================================================================
        % INVERSE GAMMA %
        logpri         = logIG1pdf(sig_igamma(f),mi,si);
        % construct the NC state space for igamma prior
        % construct the centered state space
        F = [rho, 0; 1, alpha];
        G = eye(2,1);
        H = [1, alpha];
        Voo = sig_igamma(f)^2;
        R = sigme^2;

        % initialize Kalman filter
        nx     = size(F,1);
        init_x = zeros(nx,1);
        Q      = G * Voo * G';
        P00    = inv(eye(nx^2)-kron(F,F))*reshape(Q,nx^2,1);
        init_V = reshape(P00,nx,nx);    

        % kalman filter 
        iSign  = 1;
        dConst = 0;
        [x, V, VV, loglik] = kalman_filter(y(:,e)', F, H, Q, R, dConst, init_x, init_V, iSign);    
        kernel_igamma(f)  = loglik + logpri;

        %%=================================================================
        % EXPONENTIAL PRIOR %
        logpri         = log(exppdf(sig_expo(f),lambda));
        % construct the NC state space for igamma prior
        % construct the centered state space
        F = [rho, 0; 1, alpha];
        G = eye(2,1);
        H = [1, alpha];
        Voo = sig_expo(f)^2;
        R = sigme^2;

        % initialize Kalman filter
        nx     = size(F,1);
        init_x = zeros(nx,1);
        Q      = G * Voo * G';
        P00    = inv(eye(nx^2)-kron(F,F))*reshape(Q,nx^2,1);
        init_V = reshape(P00,nx,nx);    

        % kalman filter 
        iSign  = 1;
        dConst = 0;
        [x, V, VV, loglik] = kalman_filter(y(:,e)', F, H, Q, R, dConst, init_x, init_V, iSign);    
        kernel_expo(f)  = loglik + logpri;

        %%=================================================================
        % NORMAL PRIOR (non identified signs) %
        logpri      = log(normpdf(sig_norm(f),mn,sn));
        % construct the NC state space
        F   = [rho, 0; sig_norm(f), alpha];
        G   = eye(2,1);
        H   = [sig_norm(f), alpha];
        Voo = 1;
        R   = sigme^2;
        % initialize Kalman filter
        nx     = size(F,1);
        init_x = zeros(nx,1);
        Q      = G * Voo * G';
        P00    = inv(eye(nx^2)-kron(F,F))*reshape(Q,nx^2,1);
        init_V = reshape(P00,nx,nx);    

        % kalman filter 
        [x, V, VV, loglik_nc] = kalman_filter(y(:,e)', F, H, Q, R, dConst, init_x, init_V, iSign);
        kernel_norm(f)  = loglik_nc + logpri;

    end
    
%     figure('Name',['Posteior distribution of SD technology when the true is ' num2str(sigma(e)) ])
%     plot(sig_igamma,real(kernel_igamma),'Color',colors(1,:),'Linewidth',2); hold on;
%     plot(sig_norm,kernel_norm,'Color',colors(2,:),'Linewidth',2); hold on;
%     plot(sig_expo,kernel_expo,'Color',colors(3,:),'Linewidth',2); hold on;
%     axis tight
% %     ylim([-300,0])
% %     line([sigma(e) sigma(e)],[min(kernel_norm) max(kernel_norm)],'Color',[0.75,0.75,0.75],'Linewidth',2) %'Color',gray
%     title(['\sigma = ' num2str(sigma(e)) ],'fontsize',16)
%     legend(['igamma SD' num2str(si)],['normal SD' num2str(sn)],['expo SD' num2str(lambda)])
%     axis tight
%     ext   = 'fig';
%     type  = 'fig';
%     saveas(gcf,[ './results/priorpost_true_' num2str(e) '.' ext],num2str(type))
%     ext   = 'eps';
%     type  = 'psc2';
%     saveas(gcf,[ './results/priorpost_true_' num2str(e) '.' ext],num2str(type))
    
    figure(1)
    subplot(2,2,e)
    plot(sig_igamma,real(kernel_igamma),'Color',colors(1,:),'Linewidth',2); hold on;
        axis tight
    plot(sig_norm,kernel_norm,'Color',colors(2,:),'Linewidth',2); hold on;
    plot(sig_expo,kernel_expo,'Color',colors(3,:),'Linewidth',2); hold on;
    axis tight
     ylim([-400,100])
%     line([sigma(e) sigma(e)],[min(kernel_norm) max(kernel_norm)],'Color',[0.75,0.75,0.75],'Linewidth',2) %'Color',gray
    title(['\sigma = ' num2str(sigma(e)) ],'fontsize',16)
    if e==4
        legend(['igamma SD' num2str(si)],['normal SD' num2str(sn)],['expo SD' num2str(lambda)])
    end
    axis tight
    ext   = 'fig';
    type  = 'fig';
    saveas(gcf,[ './results/priorpost.' ext],num2str(type))
    ext   = 'eps';
    type  = 'psc2';
    saveas(gcf,[ './results/priorpost.' ext],num2str(type))


end