function x = myinvnormcdf(  p )
% Inverse of the CDF of the *Standard* Normal Distribution
%  Find scalar value x s.t.
%     Pr( X <= x ) = p,  where random variable X ~ Normal( 0, 1 )
% NOTES
% - Assume x is already standardized ( e.g. x = (x-Mu)/Sigma  )
% - Supports vectorization
%   Given Nx1 vector p, will find Nx1 vector x 
%     where each entry x[i] is the inverse cdf of p[i]
% - Use the complementary error function instead erf itself
%      to produce accurate near-zero results for large negative x.
% REPLACES
%   norminv, matlab builtin function that has costly overhead in 
%            argument checking, flexible mean/variance, etc.

x = -sqrt(2).*erfcinv( 2*p );
end
