function p = mynormcdf(  x )
% CDF of the *Standard* Normal Distribution
%  Find probability p s.t.
%     Pr( X <= x ) = p,  where random variable X ~ Normal( 0, 1 )
% NOTES
% - Assume x is already standardized ( e.g. x = (x-Mu)/Sigma  )
% - Supports vectorization
%   Given Nx1 vector x, will find Nx1 vector p
%     where each entry p[i] = cdf( x[i] )
% - Use the complementary error function instead of erf itself
%      to produce accurate near-zero results for large negative x.
% REPLACES
%   normcdf, matlab builtin function that has costly overhead in 
%            argument checking, flexible mean/variance, etc.

p = 0.5 * erfc(-x ./ sqrt(2));
end
