@#define first_obs_ = 1400
@#define mh_replic_ = 00000

//@#define simul_ = ["zro","sml","med","big"]
//@#define simul_ = ["zro"]
//@#define simul_ = ["sml"]
@#define simul_ = ["med"]
//@#define simul_ = ["big"]

//=========================================================================
var
  @#for co in simul_
        // observed variables
        y_@{co}
        // capital quantities
        k_@{co}
        // innovations
        eps_a_@{co}
  @#endfor 
 ;

varexo
	eta_a;  

varexo
    // measurement errors
	em_y;  

parameters	
	alpha 
	rho_a
;

// fixed parameters
	alpha = 0.33;
	rho_a = 0.70;



// initial values 
shocks;
	var eta_a; stderr 1;
	var em_y; stderr 0.08;
end;

//=========================================================================

model(linear); 
    @#for co in simul_
        k_@{co} = alpha*k_@{co}(-1) + eps_a_@{co};    
        // Shocks
        eps_a_@{co} = rho_a * eps_a_@{co}(-1) + eta_a;
        // Measurement equations
        y_@{co} = alpha * k_@{co}(-1) + eps_a_@{co} + em_y;	
    @#endfor 
end;
//=========================================================================

varobs 
    @#for co in simul_
        y_@{co}
    @#endfor 
;

//=========================================================================
// Normal Priors //
//=========================================================================
indexstr = 0;
@#for locationStr in ["0.00","0.1","0.2","0.3","0.4"] 
    indexstr = indexstr +1;

        estimated_params;

            alpha,  0.20,0.01,1.00,     NORMAL_PDF,0.3,0.05;

            rho_a,  .70,.01,.9999,    BETA_PDF,0.5,0.20;
            
            stderr eta_a,       NORMAL_PDF,@{locationStr},5;

            stderr em_y,        INV_GAMMA_PDF,0.2,5;
        end;

        //=========================================================================
        estimation(
            nograph,
            optim=('Algorithm','active-set', 'MaxIter',500),
            datafile=simdata,
            mode_compute=1,
            mh_jscale = 0.50,   
            first_obs= @{first_obs_}, // small sample
            mh_replic = @{mh_replic_},
            mh_nblocks=2
            );

        LogMLn(indexstr) = oo_.MarginalDensity.LaplaceApproximation;
        Rslts_n(indexstr) = oo_;
    pause(2)
@#endfor


//=========================================================================
// Inverse Gamma Priors on STD //
//=========================================================================
indexstr = 0;
@#for locationStr in ["0.05","0.1","0.2","0.3","0.4"] 
    indexstr = indexstr +1;

        estimated_params;

            alpha,  0.20,0.01,1.00,     NORMAL_PDF,0.3,0.05;

            rho_a,  .70,.01,.9999,    BETA_PDF,0.5,0.20;
            
            stderr eta_a,       INV_GAMMA_PDF,@{locationStr},5;

            stderr em_y,        INV_GAMMA_PDF,0.2,5;
        end;

        //=========================================================================
        estimation(
            nograph,
            optim=('Algorithm','active-set', 'MaxIter',500),
            datafile=simdata,
            mode_compute=1,
            mh_jscale = 0.50,   
            first_obs= @{first_obs_}, // small sample
            mh_replic = @{mh_replic_},
            mh_nblocks=2
            );

        LogMLig(indexstr) = oo_.MarginalDensity.LaplaceApproximation;
        Rslts_ig(indexstr) = oo_;        
pause(2)    
@#endfor

//=========================================================================
// Exponential Priors on STD // EXP(lambda) = GAMMA(1,1/lambda)
//=========================================================================\

estimated_params;

    alpha,  0.20,0.01,1.00,     NORMAL_PDF,0.3,0.05;

    rho_a,  .70,.01,.9999,    BETA_PDF,0.5,0.20;

    stderr eta_a, .70,.001,.9999,    GAMMA_PDF,5,5;

    stderr em_y,        INV_GAMMA_PDF,0.2,5;
end;

//=========================================================================
estimation(
    nograph,
    optim=('Algorithm','active-set', 'MaxIter',500),
    datafile=simdata,
    mode_compute=1,
    mh_jscale = 0.50,    
    first_obs= @{first_obs_}, // small sample
    mh_replic = @{mh_replic_},
    mh_nblocks=2
    );

LogMLexp = oo_.MarginalDensity.LaplaceApproximation;
Rslts_exp(1) = oo_;            


tmp = 1500 - @{first_obs_};
save(['estsimdata_@{co}_' num2str(tmp)],'LogMLig','LogMLn','LogMLexp','Rslts_exp','Rslts_ig','Rslts_n');

